import { useState } from "react";
import { Link } from "wouter";
import { ArrowLeft, ExternalLink, Wallet, Download } from "lucide-react";
import { Button } from "@/components/ui/button";
import { Card } from "@/components/ui/card";
import { useToast } from "@/hooks/use-toast";
import { LazyImage } from "@/components/ui/lazy-image";
import { Header } from "@/components/ui/header";
import { useMutation } from "@tanstack/react-query";
import { apiRequest } from "@/lib/queryClient";
import { Footer } from "@/components/ui/footer";

const MODEL_TYPE = "image-generator" as const;

export default function ImageGenerator() {
  const [prompt, setPrompt] = useState("");
  const [generatedImage, setGeneratedImage] = useState<string | null>(null);
  const { toast } = useToast();

  const [negativePrompt, setNegativePrompt] = useState("");
  const [imageSize, setImageSize] = useState("1024x1024");
  const [seed, setSeed] = useState<number | null>(null);
  const [imageMeta, setImageMeta] = useState<{
    seed?: number;
    prompt: string;
  } | null>(null);
  const [showAdvanced, setShowAdvanced] = useState(false);
  const [isDownloading, setIsDownloading] = useState(false);

  const generateImage = useMutation({
    mutationFn: async (params: {
      prompt: string;
      negative_prompt?: string;
      image_size?: string;
      seed?: number;
    }) => {
      const response = await apiRequest("POST", "/api/generate-image", params);
      if (!response.ok) {
        const errorData = await response.json();
        throw new Error(errorData.details || "Failed to generate image");
      }
      return response.json();
    }
  });

  const handleSubmit = async (e: React.FormEvent) => {
    e.preventDefault();
    if (!prompt.trim()) {
      toast({
        variant: "destructive",
        title: "Error",
        description: "Please enter a prompt to generate an image.",
      });
      return;
    }

    const params: any = {
      prompt: prompt.trim()
    };

    if (negativePrompt.trim()) params.negative_prompt = negativePrompt.trim();
    if (imageSize) params.image_size = imageSize;
    if (seed !== null) params.seed = seed;

    try {
      const result = await generateImage.mutateAsync(params);
      setGeneratedImage(result.imageUrl);
      setImageMeta({
        seed: result.seed,
        prompt: result.prompt || prompt
      });
      toast({
        title: "Image Generated",
        description: "Your image has been generated successfully.",
      });
    } catch (error) {
      // Parse the error response
      let errorMessage = "Failed to generate image. Please try again.";
      let requiresToken = false;
      
      if (error instanceof Error) {
        errorMessage = error.message;
        
        // Check if this is a rate limit error
        if (error.message.includes("limit") || error.message.includes("rate")) {
          requiresToken = true;
        }
      }
      
      toast({
        variant: "destructive",
        title: requiresToken ? "Token Limit Reached" : "Error",
        description: (
          <div className="space-y-2">
            <p>{errorMessage}</p>
            {requiresToken && (
              <div className="pt-2 text-xs">
                <p className="whitespace-nowrap">To get unlimited access, hold at least 100,000 OXAI tokens.</p>
                <a 
                  href="https://www.oxai.io" 
                  target="_blank" 
                  rel="noopener noreferrer"
                  className="text-primary underline hover:opacity-80 block mt-1"
                >
                  Get OXAI tokens
                </a>
              </div>
            )}
          </div>
        ),
        duration: requiresToken ? 8000 : 5000, // Show longer for token limit messages
      });
    }
  };

  const handleRandomSeed = () => {
    setSeed(Math.floor(Math.random() * 10000000));
  };

  const clearSeed = () => {
    setSeed(null);
  };
  
  const downloadImage = async (imgUrl: string, fileName: string) => {
    try {
      setIsDownloading(true);
      
      // Direct approach: fetch the image and convert it to a local file
      const response = await fetch(imgUrl);
      if (!response.ok) {
        throw new Error(`Failed to fetch image: ${response.status} ${response.statusText}`);
      }
      
      // Get the image as a blob
      const imageBlob = await response.blob();
      
      // Create an offscreen image to load the blob
      const img = new Image();
      img.crossOrigin = "anonymous";
      
      // Create a Promise to wait for the image to load
      const imageLoaded = new Promise<void>((resolve, reject) => {
        img.onload = () => resolve();
        img.onerror = () => reject(new Error("Failed to load image"));
      });
      
      // Create object URL from the blob
      const objectUrl = URL.createObjectURL(imageBlob);
      img.src = objectUrl;
      await imageLoaded;
      
      // Create a canvas with the same dimensions as the image
      const canvas = document.createElement("canvas");
      canvas.width = img.naturalWidth;
      canvas.height = img.naturalHeight;
      
      // Draw the image on the canvas
      const ctx = canvas.getContext("2d");
      if (!ctx) {
        throw new Error("Failed to get canvas context");
      }
      ctx.drawImage(img, 0, 0);
      
      // Cleanup blob URL
      URL.revokeObjectURL(objectUrl);
      
      // Convert to PNG (explicitly set format and quality)
      const pngData = canvas.toDataURL("image/png", 1.0);
      
      // Create the final filename with proper extension
      const pngFileName = fileName.toLowerCase().endsWith('.png')
        ? fileName
        : fileName.replace(/\.[^/.]+$/, "") + ".png";
        
      // Simple approach: Create a download link with the data URI
      // This avoids CORS, sandbox, and iframe issues
      const downloadLink = document.createElement("a");
      downloadLink.href = pngData;
      downloadLink.download = pngFileName;
      downloadLink.style.display = "none";
      
      // Insert into DOM, trigger click, then remove
      document.body.appendChild(downloadLink);
      downloadLink.click();
      document.body.removeChild(downloadLink);
      
      toast({
        title: "Download Complete",
        description: "Your image has been downloaded as PNG",
        duration: 3000,
      });
    } catch (error) {
      console.error("Download failed:", error);
      toast({
        variant: "destructive",
        title: "Download Failed",
        description: "Could not download the image. Please try again.",
      });
    } finally {
      setIsDownloading(false);
    }
  };

  return (
    <div className="min-h-screen bg-background flex flex-col">
      {/* Unified Header */}
      <Header mode="image" model={MODEL_TYPE} />
      
      <main className="flex-1 pt-16">
        <div className="container mx-auto max-w-4xl px-4 sm:px-6 pt-28 sm:pt-40">
          <div className="space-y-6">
            <div className="text-center mb-6 sm:mb-12">
              <Link href="/" className="absolute left-4 sm:left-6 top-24 sm:top-28">
                <Button variant="ghost" size="icon" className="hover:bg-primary/5">
                  <ArrowLeft className="h-4 w-4" />
                </Button>
              </Link>
              <h2 className="text-2xl font-semibold mb-2">Text to Image Generator</h2>
              <p className="text-sm text-muted-foreground">
                Create stunning images from your text descriptions
              </p>
            </div>

            <Card className="p-6 bg-card">
              <form onSubmit={handleSubmit} className="space-y-4">
                <div>
                  <label htmlFor="prompt" className="block text-sm font-medium mb-1 text-foreground">
                    Prompt
                  </label>
                  <textarea
                    id="prompt"
                    value={prompt}
                    onChange={(e) => setPrompt(e.target.value)}
                    className="w-full h-24 p-3 border rounded-md focus:ring-2 focus:ring-primary/50 bg-background text-foreground border-input"
                    placeholder="Describe the image you want to generate..."
                  />
                </div>

                <div className="flex items-center space-x-2">
                  <Button
                    type="button"
                    variant="outline"
                    size="sm"
                    onClick={() => setShowAdvanced(!showAdvanced)}
                    className="flex items-center"
                  >
                    {showAdvanced ? "Hide" : "Show"} Advanced Options
                  </Button>

                  {imageMeta && (
                    <Button
                      type="button"
                      variant="outline"
                      size="sm"
                      onClick={() => {
                        if (imageMeta) {
                          setPrompt(imageMeta.prompt);
                          setSeed(imageMeta.seed ?? null);
                        }
                      }}
                    >
                      Use Last Settings
                    </Button>
                  )}
                </div>

                {showAdvanced && (
                  <div className="bg-card/50 p-4 rounded-md space-y-4">
                    <div>
                      <label htmlFor="negativePrompt" className="block text-sm font-medium mb-1 text-foreground">
                        Negative Prompt
                      </label>
                      <input
                        id="negativePrompt"
                        value={negativePrompt}
                        onChange={(e) => setNegativePrompt(e.target.value)}
                        className="w-full p-2 border rounded-md focus:ring-2 focus:ring-primary/50 bg-background text-foreground border-input"
                        placeholder="Elements to exclude from the image..."
                      />
                    </div>

                    <div className="grid grid-cols-1 md:grid-cols-2 gap-4">
                      <div>
                        <label htmlFor="imageSize" className="block text-sm font-medium mb-1 text-foreground">
                          Image Size
                        </label>
                        <select
                          id="imageSize"
                          value={imageSize}
                          onChange={(e) => setImageSize(e.target.value)}
                          className="w-full p-2 border rounded-md focus:ring-2 focus:ring-primary/50 bg-background text-foreground border-input"
                        >
                          <option value="1024x1024">1024x1024 (Square)</option>
                          <option value="1344x768">1344x768 (Landscape)</option>
                          <option value="768x1344">768x1344 (Portrait)</option>
                          <option value="1152x896">1152x896 (Wide)</option>
                          <option value="896x1152">896x1152 (Tall)</option>
                        </select>
                      </div>

                      <div>
                        <label htmlFor="seed" className="block text-sm font-medium mb-1 text-foreground">
                          Seed
                        </label>
                        <div className="flex space-x-2">
                          <input
                            id="seed"
                            type="number"
                            value={seed === null ? "" : seed}
                            onChange={(e) => setSeed(e.target.value ? parseInt(e.target.value) : null)}
                            className="w-full p-2 border rounded-md focus:ring-2 focus:ring-primary/50 bg-background text-foreground border-input"
                            placeholder="Random"
                          />
                          <Button type="button" variant="outline" size="icon" onClick={handleRandomSeed}>
                            🎲
                          </Button>
                          {seed !== null && (
                            <Button type="button" variant="outline" size="icon" onClick={clearSeed}>
                              ✕
                            </Button>
                          )}
                        </div>
                      </div>
                    </div>
                  </div>
                )}

                <div className="flex justify-start">
                  <Button
                    type="submit"
                    className="w-auto"
                    style={{ backgroundColor: "#0D11FF", color: "white" }}
                    disabled={generateImage.isPending || !prompt.trim()}
                  >
                    {generateImage.isPending ? (
                      <>
                        <span className="animate-spin mr-2">⚙️</span>
                        Generating...
                      </>
                    ) : (
                      "Generate Image"
                    )}
                  </Button>
                </div>
              </form>
            </Card>

            {generatedImage && (
              <Card className="p-6">
                <div className="aspect-square relative">
                  <LazyImage
                    src={generatedImage}
                    alt="Generated image"
                    className="w-full h-full object-contain rounded-lg"
                  />
                </div>
                {imageMeta && (
                  <div className="mt-4 bg-muted/30 p-4 rounded-md text-sm">
                    <h3 className="font-medium mb-2">Image Details</h3>
                    {imageMeta.seed && <p><strong>Seed:</strong> {imageMeta.seed}</p>}
                    {imageMeta.prompt && <p><strong>Prompt:</strong> {imageMeta.prompt}</p>}
                  </div>
                )}
                
                <div className="mt-4 flex flex-col sm:flex-row gap-3">
                  <Button
                    className="w-full sm:w-auto h-10 py-2 px-4"
                    style={{ backgroundColor: "#0D11FF", color: "white" }}
                    onClick={() => downloadImage(generatedImage, "generated-image.png")}
                    disabled={isDownloading}
                  >
                    {isDownloading ? (
                      <>
                        <span className="animate-spin mr-2">⚙️</span>
                        Downloading...
                      </>
                    ) : (
                      <>
                        <Download className="h-4 w-4 mr-2" />
                        Download Image
                      </>
                    )}
                  </Button>
                  
                  <Button
                    className="w-full sm:w-auto h-10 py-2 px-4 pr-28 group relative flex items-center cursor-not-allowed hover:bg-[#FFB20D]/10 transition-colors"
                    variant="outline"
                  >
                    <div className="flex items-center gap-2">
                      <Wallet className="h-4 w-4" />
                      Deploy as an NFT on Oxai Network
                    </div>
                    <span className="text-xs bg-yellow-500/10 text-yellow-500 px-2 py-0.5 rounded absolute right-4 top-1/2 -translate-y-1/2">
                      Coming Soon
                    </span>
                  </Button>
                </div>
              </Card>
            )}
          </div>
        </div>
      </main>
      
      <footer className="mt-16 sm:mt-8">
        <Footer />
      </footer>
    </div>
  );
}